import flet as ft
from calet.calet_theme import ClTheme

class ClInputField(ft.UserControl):
    
    def __init__(self, theme:ClTheme, label:str=None, placeholder:str=None, max_length:int=None,
                 max_lines:int=None, min_lines:int=None, multiline:bool=False, expand:bool|int=False,
                 visible:bool=True, default:str=None):

        super().__init__()
        self.theme = theme
        self.label = label
        self.placeholder = placeholder
        self.max_length = max_length
        self.max_lines = max_lines
        self.min_lines = min_lines
        self.multiline = multiline
        self.expand = expand
        self.visible = visible
        self.default = default
    
    def build(self):

        # FIELD
        # - campo de entrada
        self.field = ft.TextField(
            value=self.default,
            label=self.label,
            label_style=ft.TextStyle(
                color=self.theme.font_one,
                size=14,
            ),
            hint_text=self.placeholder,
            hint_style=ft.TextStyle(
                color=self.theme.font_one,
                size=14,
            ),
            counter_style=ft.TextStyle(color=self.theme.font_one),
            content_padding=10,
            border_radius=10,
            border_width=1,
            focused_border_width=2,
            color=self.theme.font_one,
            focused_color=self.theme.font_two,
            border_color=self.theme.divider,
            focused_border_color=self.theme.primary,
            bgcolor=self.theme.transparent,
            focused_bgcolor=self.theme.transparent_05,
            text_align=ft.TextAlign.LEFT,
            max_length=self.max_length,
            max_lines=self.max_lines,
            min_lines=self.min_lines,
            multiline=self.multiline,
            on_focus=self.tf_focused,
            on_blur=self.tf_blurred
        )

        return self.field
    
    # EVENT HANDLER METHODS
    def tf_focused(self, e:ft.ControlEvent):
        e.control.label_style = ft.TextStyle(
            color=self.theme.font_two,
            size=14,
        )
        if self.field.error_text is not None:
            self.field.error_text = None
        self.update()
    
    def tf_blurred(self, e:ft.ControlEvent):
        e.control.label_style = ft.TextStyle(
            color=self.theme.font_one,
            size=14,
        )
        self.update()
    
    # SUPPORT METHODS
    def error(self, error:str):
        """Change the input field to error status.\n
        ---
        - error: Is the text to display in the error message. 
        """
        self.field.error_text = error
        self.field.error_style = ft.TextStyle(
            color=self.theme.error,
        )
        self.update()

class ClPasswordInput(ClInputField):

    def build(self):
        super().build()

        # CONFIGURACION PARA CAMPO DE CONTRASEÑA
        self.field.password = True
        self.field.can_reveal_password = True

        return self.field