import flet as ft
from calet.calet_theme import ClTheme, ClDarkTheme, ClLightTheme
from calet.calet_bar import ClAppBar, ClMenuBar, ClMenuSection
from calet.calet_button import *
from calet.calet_input import *
from sumet_edition import SmImage, SmImagesViewer, SmImagesEditor

class SuMetApp(ft.UserControl):
    
    def __init__(self):
        """Crea una instancia de la app. Esta se encarga de manejar todo
        el flujo principal del programa."""

        super().__init__()
        self.theme = ClTheme(on_light=ClLightTheme(), on_dark=ClDarkTheme(), mode="dark")
    
    def build(self):

        # BARRA DE TITULO DE LA APP
        # - boton "buscar imagen"
        self.b_search_images = ClTextButton(
            theme=self.theme,
            icon=ft.icons.IMAGE_SEARCH_OUTLINED,
            hover_icon=ft.icons.IMAGE_SEARCH,
            content_size=12,
            width=40,
            action=self.b_search_images_clicked
        )
        # - barra
        self.app_bar = ClAppBar(
            theme=self.theme,
            title="SuMetApp",
            content_size=14,
            left_actions=[self.b_search_images],
            win_actions=[
                ClWinButton(
                    theme=self.theme,
                    winaction="minimize"
                ),
                ClWinButton(
                    theme=self.theme,
                    winaction="close"
                )
            ],
        )

        # BARRA DE MENU
        # - boton 'descartar todas las imagenes'
        self.b_discard_all = ClOptionButton(
            theme=self.theme,
            text="Descartar todas",
            icon=ft.icons.DELETE,
            content_size=12,
            height=30,
            action=self.b_discard_all_clicked
        )
        # - boton 'descartar imagen'
        self.b_discard = ClMenuButton(
            theme=self.theme,
            main_button=ClTextButton(
                theme=self.theme,
                text="Descartar",
                icon=ft.icons.DELETE,
                content_size=12,
                action=self.b_discard_clicked
            ),
            options=[self.b_discard_all]
        )
        # - boton 'editar imagen'
        self.b_edit = ClTextButton(
            theme=self.theme,
            text="Editar",
            icon=ft.icons.EDIT,
            content_size=12,
            action=self.b_edit_clicked
        )
        # - barra de menu
        self.app_menu_bar = ClMenuBar(
            theme=self.theme,
            bar_size=0,
            defined=True,
            sections=[
                ClMenuSection(
                    theme=self.theme,
                    actions=[[self.b_discard]]
                ),
                ClMenuSection(
                    theme=self.theme,
                    actions=[[self.b_edit]]
                ),
            ]
        )

        # BARRA DE MENU DEL MODO EDICION
        # - boton 'deshacer edicion'
        self.b_undo_edition = ClTextButton(
            theme=self.theme,
            icon=ft.icons.UNDO,
            content_size=12,
            width=30,
            action=self.b_undo_edition_clicked
        )
        # - boton 'confirmar ediciones'
        self.b_confirm_editions = ClTextButton(
            theme=self.theme,
            text="Terminar ediciones",
            icon=ft.icons.CHECK,
            content_size=12,
            action=self.b_confirm_editions_clicked
        )
        # - boton 'cancelar ediciones'
        self.b_cancel_editions = ClTextButton(
            theme=self.theme,
            text="Cancelar ediciones",
            icon=ft.icons.CANCEL_OUTLINED,
            content_size=12,
            action=self.b_cancel_editions_clicked
        )
        # - barra de menu de la app para el modo edicion
        self.edition_menu_bar = ClMenuBar(
            theme=self.theme,
            bar_size=0,
            defined=True,
            sections=[
                ClMenuSection(
                    theme=self.theme,
                    actions=[[self.b_undo_edition]]
                ),
                ClMenuSection(
                    theme=self.theme,
                    actions=[[self.b_confirm_editions],[self.b_cancel_editions]]
                )
            ]
        )
        
        # SECCION ABIERTA DE LA APP
        # - seccion abierta
        self.app_section = SmMainSection(self.theme, self.b_search_images_clicked)

        # APLICACION
        # - contenido de capa principal
        self.main_overlay_content = ft.Container(
            expand=True,
            bgcolor=self.theme.background_one,
            content=ft.Column(
                spacing=0,
                controls=[self.app_menu_bar, self.edition_menu_bar, self.app_section]
            )
        )
        # - capa principal
        self.main_overlay = ft.Container(
            expand=True,
            content=ft.Column(
                spacing=0,
                controls=[
                    self.app_bar,
                    self.main_overlay_content
                ]
            )
        )
        # - app
        self.app = ft.Stack(
            expand=True,
            controls=[
                self.main_overlay
            ]
        )

        return self.app
    
    # METODOS MANEJADORES DE EVENTOS
    def b_search_images_clicked(self, e:ft.TapEvent):
        self.fpk_dialog = ft.FilePicker(
            on_result=self.fpk_img_selector_resolved
        )
        self.page.overlay.append(self.fpk_dialog)
        self.page.update()
        self.fpk_dialog.pick_files(
            dialog_title="Selecciona una o varias fotos para trabajar...",
            file_type=ft.FilePickerFileType.IMAGE,
            allow_multiple=True
        )
    
    def fpk_img_selector_resolved(self, e:ft.FilePickerResultEvent):
        if e.files is not None:
            images = [SmImage(file.name, file.path) for file in e.files]
            if isinstance(self.app_section, SmMainSection):
                self.app_section = SmImagesSection(
                    theme=self.theme,
                    images=images,
                    upd_app=self.upd_app,
                )
                self.main_overlay_content.content.controls[-1] = self.app_section
                self.app_menu_bar.upd(bar_size=40)
                self.app_bar.upd(title=f"SuMetApp - {images[0].controller.name}")
            elif isinstance(self.app_section, SmImagesSection):
                self.app_section.viewer.add_images(images)
            self.update()
        self.page.overlay.remove(self.fpk_dialog)
        self.page.update()

    def b_discard_clicked(self, e:ft.TapEvent):
        self.app_section.viewer.discard_images()
        if not self.app_section.viewer.controller.has_images():
            self.app_section = SmMainSection(
                theme=self.theme,
                search_action=self.b_search_images_clicked
            )
            self.main_overlay_content.content.controls[-1] = self.app_section
            self.app_menu_bar.upd(bar_size=0)
            self.app_bar.upd(title="SuMetApp")
        self.update()
    
    def b_discard_all_clicked(self, e:ft.TapEvent):
        self.app_section.viewer.discard_images(all=True)
        self.app_section = SmMainSection(
            theme=self.theme,
            search_action=self.b_search_images_clicked
        )
        self.main_overlay_content.content.controls[-1] = self.app_section
        self.app_menu_bar.upd(bar_size=0)
        self.app_bar.upd(title="SuMetApp")
        self.update()

    def b_edit_clicked(self, e:ft.TapEvent):
        self.app_menu_bar.upd(bar_size=0)
        self.edition_menu_bar.upd(bar_size=40)
        self.app_section.viewer.lock_carrousel()
        self.app_section.editor.start_edition(self.app_section.viewer.showed_image())
        self.b_search_images.visible = False
        self.b_search_images.update()
        self.update()

    def b_undo_edition_clicked(self, e:ft.TapEvent):
        self.app_section.editor.undo_edition()

    def b_confirm_editions_clicked(self, e:ft.TapEvent):
        self.app_menu_bar.upd(bar_size=40)
        self.edition_menu_bar.upd(bar_size=0)
        self.app_section.editor.confirm_edition()
        self.app_section.viewer.unlock_carrousel()
        self.b_search_images.visible = True
        self.b_search_images.update()
        self.update()

    def b_cancel_editions_clicked(self, e:ft.TapEvent):
        self.app_menu_bar.upd(bar_size=40)
        self.edition_menu_bar.upd(bar_size=0)
        self.app_section.editor.cancel_edition()
        self.app_section.viewer.unlock_carrousel()
        self.b_search_images.visible = True
        self.b_search_images.update()
        self.update()

    # METODOS DE ACCION
    def upd_app(self, title:str=None):
        
        if title is not None:
            self.app_bar.upd(title=title)
        self.update()

    def open_dialog(self, dlg):
        self.app.controls.insert(-1, dlg)
        self.update()
    
    def close_dialog(self, dlg):
        self.app.controls.remove(dlg)
        self.update()

class SmSection(ft.UserControl):
    
    def __init__(self, theme:ClTheme):

        super().__init__()
        self.theme = theme
        self.expand = True
    
    def build(self):

        # SECCION
        self.section = ft.Container(
            bgcolor=self.theme.background_two,
            border_radius=10,
            margin=5,
            padding=5,
            alignment=ft.alignment.center
        )

        return self.section

class SmMainSection(SmSection):

    def __init__(self, theme:ClTheme, search_action):
        super().__init__(theme)
        self.search_action = search_action
    
    def build(self):

        super().build()

        # CONTENIDO DE LA SECCION PRINCIPALS
        # - boton de acceso directo a 'buscar imagenes'
        self.b_search_images = ClTextButton(
            theme=self.theme,
            text="Busca una imagen para comenzar...",
            icon=ft.icons.IMAGE_SEARCH_OUTLINED,
            content_size=20,
            content_padding=10,
            radius=10,
            action=self.search_action
        )
        # - seccion
        self.section.content = ft.Row(
            spacing=0,
            alignment=ft.MainAxisAlignment.CENTER,
            controls=[
                ft.Column(expand=True),
                self.b_search_images,
                ft.Column(expand=True)
            ]
        )

        return self.section

class SmImagesSection(SmSection):

    def __init__(self, theme:ClTheme, images:list[SmImage], upd_app):
        super().__init__(theme)
        self.images = images
        self.upd_app = upd_app

    def build(self):

        super().build()

        # CONTENIDO DE LA SECCION DE IMAGENES
        # - visor de fotos
        self.viewer = SmImagesViewer(
            theme=self.theme, 
            images=self.images, 
            upd_app=self.upd_app
        )
        # - editor
        self.editor = SmImagesEditor(
            theme=self.theme, 
        )
        # - seccion
        self.section.content = ft.Stack(
            controls=[
                self.viewer,
                self.editor
            ]
        )

        return self.section
    
